@@ -1,6 +1,6 @@
 # $Author: ddumont $
-# $Date: 2009-03-31 18:20:13 +0200 (Tue, 31 Mar 2009) $
-# $Revision: 911 $
+# $Date: 2009-05-29 14:40:50 +0200 (ven 29 mai 2009) $
+# $Revision: 965 $
 
 #    Copyright (c) 2008-2009 Dominique Dumont.
 #
@@ -27,12 +27,13 @@ use warnings ;
 use Config::Model::Exception ;
 use UNIVERSAL ;
 use File::Path;
+use Log::Log4perl qw(get_logger :levels);
 
 my $has_augeas = 1;
 eval { require Config::Augeas ;} ;
 $has_augeas = 0 if $@ ;
 
-our $VERSION = '0.106';
+our $VERSION = '0.107';
 
 =head1 NAME
 
@@ -47,7 +48,7 @@ Config::Model::Backend::Augeas - Read and write config data through Augeas
 
    # try Augeas and fall-back with custom method
    read_config  => [ { backend => 'augeas' , 
-                       config_file => '/etc/ssh/sshd_config',
+                       file => '/etc/ssh/sshd_config',
                        # declare "seq" Augeas elements 
                        sequential_lens => [/AcceptEnv AllowGroups [etc]/],
                      },
@@ -81,9 +82,9 @@ Use C<augeas> (or C<Augeas>)in this case.
 Either C<backup> or C<newfile>. See L<Config::Augeas/Constructor> for
 details.
 
-=item config_file
+=item file
 
-Name of the config_file.
+Name of the configuration file.
 
 =item sequential_lens
 
@@ -97,7 +98,7 @@ For instance:
 
    read_config  => [ { backend => 'augeas' , 
                        save   => 'backup',
-                       config_file => '/etc/ssh/sshd_config',
+                       file => '/etc/ssh/sshd_config',
                        # declare "seq" Augeas elements 
                        sequential_lens => [/AcceptEnv AllowGroups/],
                      },
@@ -231,7 +232,12 @@ sub read
     $self->{augeas_obj} ||= Config::Augeas->new(root => $args{root}, 
 						save => $args{save} ) ;
 
-    foreach my $param (qw/config_dir config_file/) {
+    if (defined $args{config_file}) { 
+	warn "Augeas::read : deprecated config_file parameter, use file instead\n";
+	$args{file}||= delete $args{config_file} ; 
+    }
+
+    foreach my $param (qw/config_dir file/) {
 	if (not defined $args{$param}) {
 	    Config::Model::Exception::Model -> throw
 		(
@@ -243,11 +249,11 @@ sub read
     }
 
     my $cdir = $args{root}.$args{config_dir} ;
-    print "Read config data through Augeas in directory '$cdir' ",
-      "file $args{config_file}\n" 
-	if $::verbose;
+    my $logger =  get_logger('Data::Read') ;
+    $logger->info( "Read config data through Augeas in directory '$cdir' ".
+		   "file $args{file}");
 
-    my $mainpath = '/files'.$args{config_dir}.$args{config_file} ;
+    my $mainpath = '/files'.$args{config_dir}.$args{file} ;
 
     my @result =  $self->augeas_deep_match($mainpath) ;
     my @cm_path = @result ;
@@ -278,7 +284,8 @@ sub read
 	my $v = $augeas_obj->get($aug_p) ;
 	next unless defined $v ;
 
-	print "read-augeas $aug_p, will set C::M path $cm_p with $v\n" if $::debug ;
+	$logger->debug("read-augeas $aug_p, will set C::M path $cm_p with $v");
+
 	$cm_p =~ s!^/!! ;
 	# With some list, we can get
 	# /files/etc/ssh/sshd_config/AcceptEnv[1]/1/ =  LC_PAPER
@@ -302,9 +309,9 @@ sub read
 	while (my $step = shift @cm_steps) {
 	    my ($label,$idx) = ( $step =~ /(\w+)(?:\[(\d+)\])?/ ) ;
 	    my $is_seq = $is_seq_lens{$label} || 0 ;
-	    print "read-augeas: step label $label ",
-	      defined $idx ? "idx $idx ": '', "(is_seq $is_seq)\n"
-		if $::debug ; 
+	    $logger
+	      ->debug("read-augeas: step label $label ".
+		      (defined $idx ? "idx $idx ": '') . "(is_seq $is_seq)");
 
 	    # idx will be treated next iteration if needed
 	    if (    $obj->get_type eq 'node' 
@@ -312,7 +319,8 @@ sub read
 		and $is_seq
 	       ) {
 		$idx = 1 unless defined $idx ;
-		print "read-augeas: keep seq lens idx $idx\n" if $::debug ; 
+		$logger
+		  ->debug("read-augeas: keep seq lens idx $idx") ; 
 		unshift @cm_steps , $idx ;
 	    }
 
@@ -329,12 +337,12 @@ sub read
 	    # augeas list begin at 1 not 0
 	    $label -= 1 if $obj->get_type eq 'list';
 	    if (scalar @cm_steps > 0) {
-		print "read-augeas: get $label\n" if $::debug;
+		$logger ->debug("read-augeas: get $label");
 		$obj = $obj->get($label) ;
 	    }
 	    else {
 		# last step
-		print "read-augeas: set $label $v\n" if $::debug ; 
+		$logger->debug("read-augeas: set $label $v"); 
 		$obj->set($label,$v) ;
 	    }
 
@@ -359,14 +367,18 @@ sub augeas_deep_match {
     # level 
     # See https://www.redhat.com/archives/augeas-devel/2008-July/msg00016.html
     my @worklist = ( $mainpath );
-    print "read-augeas on @worklist\n" if $::debug ;
+    my $logger = get_logger('Data::Read') ;
+    $logger ->debug("read-augeas on @worklist") ;
 
     my $augeas_obj = $self->{augeas_obj} ;
     my @result ;
     while (@worklist) {
 	my $p = pop @worklist ;
-	my @newpath = $augeas_obj -> match($p . "/*") ;
-	print "read-augeas $p/* matches paths: @newpath\n" if $::debug ;
+	# filter out comments 
+	# see http://augeas.net/page/Path_expressions
+	my @newpath = $augeas_obj -> match($p . "/*[label() != '#comment']") ;
+	$logger
+	  ->debug("read-augeas $p/* matches paths: @newpath") ;
 	push @worklist, @newpath ;
 	push @result,   @newpath ;
     }
@@ -374,12 +386,23 @@ sub augeas_deep_match {
     return @result ;
 }
 
+# this is a bit of a hack. This function is called by Autoread to
+# check whether the configuration file should be opened before calling
+# write or not. If the config file is opened before augeas writes in
+# it, all comments and structure is lost.
+sub skip_open { 1;}
+
 sub write {
     my $self = shift;
     my %args = @_ ; # contains root and config_dir
     return 0 unless $has_augeas ;
 
-    foreach my $param (qw/config_dir config_file/) {
+    if (defined $args{config_file}) { 
+	warn "Augeas::write : deprecated config_file parameter, use file instead\n";
+	$args{file}||= delete $args{config_file} ; 
+    }
+
+    foreach my $param (qw/config_dir file/) {
 	if (not defined $args{$param}) {
 	    Config::Model::Exception::Model -> throw
 		(
@@ -391,12 +414,12 @@ sub write {
     }
 
     my $cdir = $args{root}.$args{config_dir} ;
-    print "Write config data through Augeas in directory '$cdir' ",
-      "file $args{config_file}\n" 
-	if $::verbose;
+    get_logger("Data::Write")
+      ->info("Write config data through Augeas in directory '$cdir' ".
+	     "file $args{file}");
 
     my $set_in = $args{set_in} || '';
-    my $mainpath = '/files'.$args{config_dir}.$args{config_file} ;
+    my $mainpath = '/files'.$args{config_dir}.$args{file} ;
     my $augeas_obj =   $self->{augeas_obj} 
                    ||= Config::Augeas->new(root => $args{root}, 
 					   save => $args{save} ) ;
@@ -411,7 +434,8 @@ sub save {
     my $self = shift ;
     my $mainpath = shift ;
 
-    $self->{augeas_obj}->print() if $::debug ;
+    # can't use augeas print directly in logging system...
+    $self->{augeas_obj}->print() if get_logger('Data::Write')->is_debug ;
 
     $self->{augeas_obj}->save || die "Augeas save failed" . 
       $self->{augeas_obj}->print("/augeas/$mainpath/*");
@@ -454,7 +478,7 @@ sub std_cb {
 
     my $v = $value_obj->fetch () ; 
     if (defined $v and $v ne '') {
-	print "copy_in_augeas: set $p = '$v'\n" if $::verbose;
+	get_logger("Data::Write")->info("copy_in_augeas: set $p = '$v'");
 	$augeas_obj->set($p , $v) ;
 	#$self->save($mainpath) if $::debug ;
     }
@@ -473,10 +497,12 @@ sub list_element_cb {
     # corresponding hash-like keys in Augeas tree
 
     # find Augeas nodes matching this path
-    my @matches = $augeas_obj->match($p) ;
+    my @matches = $augeas_obj->match($p."[label() != '#comment']") ;
 
     # need to find 2nd levels of sub-nodes for non-seq list lenses
-    @matches = sort map { $augeas_obj->match($_.'/*') ; } @matches 
+    @matches = sort map { 
+	$augeas_obj->match($_."/*[label() != '#comment']") ; 
+    } @matches 
       unless $is_seq ;
 
     # Depending on the syntax, list can be in the form:
@@ -493,8 +519,9 @@ sub list_element_cb {
 	map { s/$element_name(?!\[)/$replace/ } @matches ;
     }
 
-    print "copy_in_augeas: List (@idx) path $p matches (seq $is_seq):\n\t",
-      join("\n\t",@matches),"\n" if $::debug;
+    my $logger = get_logger("Data::Write") ;
+    $logger->debug("copy_in_augeas: List (@idx) path $p matches (seq $is_seq):\n\t".
+	      join("\n\t",@matches));
 
     # store list indexes found in Augeas and their corresponding path
     my %match = map { 
@@ -508,7 +535,7 @@ sub list_element_cb {
     # tree. Create a new Augeas path for sequential list lenses. This
     # path will be used by scan_list
     if ($is_seq) {
-	my $count = $augeas_obj->count_match($p) ;
+	my $count = $augeas_obj->count_match($p."[label() != '#comment']") ;
 	foreach my $idx (@idx) {
 	    # augeas index starts at 1 not 0
 	    my $i = $idx + 1; 
@@ -516,8 +543,8 @@ sub list_element_cb {
 
 	    $match{$i} = $p.'['.++$count."]" . ($is_seq ? '' : "/$i");
 
-	    print "copy_in_augeas: New list path $match{$i} for index $i\n"
-	      if $::debug;
+	    $logger->debug("copy_in_augeas: New list path $match{$i} "
+			   ."for index $i");
 	} 
     }
 
@@ -527,8 +554,7 @@ sub list_element_cb {
 	# use Augeas path (given by match command) or the path
 	# created for unknown non-seq list elements
 	my $scan_path = delete $match{$i+1} || $p.'/'.($i+1);
-	print "copy_in_augeas: scan list called on $scan_path index $i\n"
-	      if $::debug;
+	$logger->debug("copy_in_augeas: scan list called on $scan_path index $i");
 	$scanner->scan_list([$scan_path,$augeas_obj,$set_in,$is_seq_lens], 
 			    $node,$element_name,$i);
     }
@@ -537,17 +563,16 @@ sub list_element_cb {
     foreach (keys %match) {
 	my $rm_path = $match{$_} ;
 
-	print "copy_in_augeas: List rm $_ ->$rm_path\n" if $::debug; 
+	$logger->debug("copy_in_augeas: List rm $_ ->$rm_path"); 
 	$augeas_obj->remove($rm_path) || die "remove $rm_path failed";
 
 	# check if removing parent node in Augeas is needed
 	$rm_path =~ s!/([\w\[\]\-]+)$!! ; # chomp last "step" of the path
-	if ($augeas_obj->count_match($rm_path) == 1
+	if ($augeas_obj->count_match($rm_path."[label() != '#comment']") == 1
 	    and $set_in ne $element_name 
 	    and $rm_path =~ /$element_name$/
 	   ) {
-	    print "copy_in_augeas: List rm parent node $_ ->$rm_path\n" 
-	      if $::debug; 
+	    $logger->debug("copy_in_augeas: List rm parent node $_ ->$rm_path");
 	    $augeas_obj->remove($rm_path) || die "remove $rm_path failed";
 	}
     }
@@ -563,10 +588,12 @@ sub hash_element_cb {
     # corresponding hash-like keys in Augeas tree
 
     # find Augeas nodes matching this path
-    my @matches = $augeas_obj->match($p) ;
+    my @matches = $augeas_obj->match($p."[label() != '#comment']") ;
 
     # need to find 2nd levels of sub-nodes 
-    @matches = sort map { $augeas_obj->match($_.'/*') ; } @matches ;
+    @matches = sort map { 
+	$augeas_obj->match($_."/*[label() != '#comment']") ; 
+    } @matches ;
 
     # sequential lens need a list index to store element.  I.e
     # foo[1]/key1 foo[2]/key2 is ok. foo/key1 foo/key2 will fail But
@@ -577,8 +604,9 @@ sub hash_element_cb {
 	map { s/$element_name(?!\[)/$replace/ } @matches ;
     }
 
-    print "copy_in_augeas: Hash path $p matches (seq $is_seq):\n\t", 
-      join("\n\t",@matches),"\n" if $::debug;
+    my $logger = get_logger('Data::Write') ;
+    $logger->debug("copy_in_augeas: Hash path $p matches (seq $is_seq):\n\t". 
+		   join("\n\t",@matches));
 
     # store indexes found in Augeas and their corresponding path
     my %match = map { 
@@ -593,18 +621,17 @@ sub hash_element_cb {
     # path will be used by scan_list. This insertion cannot be done if
     # no elements are already present in Augeas tree.
     if ($is_seq and @matches) {
-	my $count = $augeas_obj->count_match($p) ;
+	my $count = $augeas_obj->count_match($p."[label() != '#comment']") ;
 	foreach (@keys) {
 	    next if defined $match{$_} ;
 
 	    my $ip = $p.'[last()]';
-	    print "inserting $element_name after $ip\n" if $::debug ;
+	    $logger->debug("inserting $element_name after $ip\n");
 	    $augeas_obj->insert($element_name, after => $ip ) 
 	      || die "augeas insert $element_name after $ip failed";
 
 	    my $np = $match{$_} = $p.'['.++$count."]/$_";
-	    print "copy_in_augeas: New hash path $np for key $_\n"
-	      if $::debug;
+	    $logger->debug("copy_in_augeas: New hash path $np for key $_");
 	} 
     }
 
@@ -621,16 +648,15 @@ sub hash_element_cb {
     # cleanup keys found in Augeas but not in Config::Model
     foreach (keys %match) {
 	my $rm_path = $match{$_} ;
-	print "copy_in_augeas: Hash rm $_ ->$rm_path\n" if $::debug; 
+	$logger->debug("copy_in_augeas: Hash rm $_ ->$rm_path"); 
 	$augeas_obj->remove($rm_path) || die "remove $rm_path failed";
 
 	# check if removing parent node in Augeas is needed
 	$rm_path =~ s!/([\w\[\]\-]+)$!! ;
-	if (    $augeas_obj->count_match($rm_path) == 1 
+	if (    $augeas_obj->count_match($rm_path."[label() != '#comment']") == 1 
 	    and $set_in ne $element_name and $is_seq
 	   ) {
-	    print "copy_in_augeas: Hash rm parent $_ ->$rm_path\n" 
-	      if $::debug; 
+	    $logger->debug("copy_in_augeas: Hash rm parent $_ ->$rm_path");
 	    $augeas_obj->remove($rm_path) || die "remove $rm_path failed";
 	}
     }
@@ -640,20 +666,22 @@ sub node_content_cb {
     my ($scanner, $data_ref,$node,@element) = @_ ;
     my ($p,$augeas_obj,$set_in,$is_seq_lens) = @$data_ref ;
 
+    my $logger = get_logger('Data::Write') ;
+
     # See set_in parameter (who said kludge ?)
     if (scalar @element == 1 and $element[0] eq $set_in) {
 	# Augeas tree is stored below element[0]
-	print "copy_in_augeas: Augeas tree set in node path $p\n" if $::debug;
+	$logger->debug("copy_in_augeas: Augeas tree set in node path $p");
 	$scanner->scan_element([$p,$augeas_obj,$set_in,$is_seq_lens], 
 			       $node,$element[0]);
     }
     else {
-	my @matches = $augeas_obj->match($p.'/*') ;
+	my @matches = $augeas_obj->match($p."/*[label() != '#comment']") ;
 	# cleanup indexes are we don't handle them now with element
 	# (later in lists and hashes)
 	map { s/\[\d+\]+$//;  } @matches ;
-	print "copy_in_augeas: Node path $p matches:\n\t", 
-	  join("\n\t",@matches),"\n" if $::debug;
+	$logger->debug("copy_in_augeas: Node path $p matches:\n\t". 
+		       join("\n\t",@matches),);
 
 	# store elements found in Augeas and their corresponding path
 	my %match = map { 
@@ -679,13 +707,12 @@ sub node_content_cb {
                           ? (after  => $p.'/'.$previous_match.'[last()]')
 		          : (before => $matches[0]     ) ;
 
-		    print "inserting $_ $direction $ip\n" if $::debug ;
+		    $logger->debug("inserting $_ $direction $ip");
 		    $augeas_obj->insert($_, $direction => $ip ) 
 		      || die "augeas insert $_ $direction $ip failed";
 
 		    my $np = $match{$_} = "$p/$_";
-		    print "copy_in_augeas: New hash path $np for element $_\n"
-		      if $::debug;
+		    $logger->debug("copy_in_augeas: New hash path $np for element $_");
 		}
 	    } 
 	}
@@ -696,8 +723,7 @@ sub node_content_cb {
 	    # use Augeas path (given by match command) or the path
 	    # created for new elements
 	    my $scan_path = delete $match{$_} || $p.'/'.$_ ;
-	    print "copy_in_augeas: Node scan $scan_path for element $_\n" 
-	      if $::debug;
+	    $logger->debug("copy_in_augeas: Node scan $scan_path for element $_");
 	    $scanner->scan_element([$scan_path,$augeas_obj,$set_in,$is_seq_lens],
 				   $node,$_)
 	}
@@ -705,7 +731,7 @@ sub node_content_cb {
 	# cleanup keys found in Augeas but not in Config::Model
 	foreach (keys %match) {
 	    my $rm_path = $match{$_} ;
-	    print "copy_in_augeas: Node rm $_ ->$rm_path\n" if $::debug; 
+	    $logger->debug("copy_in_augeas: Node rm $_ ->$rm_path");
 	    $augeas_obj->remove($rm_path) || die "remove $rm_path failed";
 	}
     }
@@ -714,6 +740,21 @@ sub node_content_cb {
 
 1;
 
+=head1 Log and trace
+
+This module use L<Log::Log4perl> to log debug and info trace with
+C<Data::Read> and C<Data::Write> categories.
+
+=head1 CAVEATS
+
+=over
+
+=item *
+
+Augeas C<#comment> nodes are ignored
+
+=back
+
 =head1 SEE ALSO
 
 =over 
